@@ -25,19 +25,21 @@
  */
 
 #include "config.h"
-#include <X11/Xlib.h>
-#include <X11/Xatom.h>
-#include <string.h>
 
-#include "gdk.h"          /* For gdk_error_trap_push/pop() */
-#include "gdkx.h"
 #include "gdkproperty.h"
+
+#include "gdkmain.h"
+#include "gdkx.h"
 #include "gdkprivate.h"
 #include "gdkinternals.h"
 #include "gdkdisplay-x11.h"
 #include "gdkscreen-x11.h"
-#include "gdkselection.h"	/* only from predefined atom */
-#include "gdkalias.h"
+#include "gdkselection.h"
+
+#include <X11/Xlib.h>
+#include <X11/Xatom.h>
+#include <string.h>
+
 
 static GPtrArray *virtual_atom_array;
 static GHashTable *virtual_atom_hash;
@@ -563,7 +565,7 @@ gdk_property_get (GdkWindow   *window,
   if (GDK_WINDOW_DESTROYED (window))
     return FALSE;
 
-  display = gdk_drawable_get_display (window);
+  display = gdk_window_get_display (window);
   xproperty = gdk_x11_atom_to_xatom_for_display (display, property);
   if (type == GDK_NONE)
     xtype = AnyPropertyType;
@@ -703,7 +705,7 @@ gdk_property_change (GdkWindow    *window,
 
   gdk_window_ensure_native (window);
 
-  display = gdk_drawable_get_display (window);
+  display = gdk_window_get_display (window);
   xproperty = gdk_x11_atom_to_xatom_for_display (display, property);
   xtype = gdk_x11_atom_to_xatom_for_display (display, type);
   xwindow = GDK_WINDOW_XID (window);
@@ -757,6 +759,3 @@ gdk_property_delete (GdkWindow *window,
 		   gdk_x11_atom_to_xatom_for_display (GDK_WINDOW_DISPLAY (window),
 						      property));
 }
-
-#define __GDK_PROPERTY_X11_C__
-#include "gdkaliasdef.c"
