@@ -23,16 +23,17 @@
 
 #include "config.h"
 
-#include <stdlib.h>
-#include <string.h>
-
-#include <glib.h>
-#include "gdkscreen.h"
 #include "gdkscreen-x11.h"
+
+#include "gdkscreen.h"
 #include "gdkdisplay.h"
 #include "gdkdisplay-x11.h"
 #include "gdkx.h"
-#include "gdkalias.h"
+
+#include <glib.h>
+
+#include <stdlib.h>
+#include <string.h>
 
 #include <X11/Xatom.h>
 
@@ -238,50 +239,6 @@ gdk_screen_get_root_window (GdkScreen *screen)
   return GDK_SCREEN_X11 (screen)->root_window;
 }
 
-/**
- * gdk_screen_get_default_colormap:
- * @screen: a #GdkScreen
- *
- * Gets the default colormap for @screen.
- * 
- * Returns: (transfer none): the default #GdkColormap.
- *
- * Since: 2.2
- **/
-GdkColormap *
-gdk_screen_get_default_colormap (GdkScreen *screen)
-{
-  g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
-
-  return GDK_SCREEN_X11 (screen)->default_colormap;
-}
-
-/**
- * gdk_screen_set_default_colormap:
- * @screen: a #GdkScreen
- * @colormap: a #GdkColormap
- *
- * Sets the default @colormap for @screen.
- *
- * Since: 2.2
- **/
-void
-gdk_screen_set_default_colormap (GdkScreen   *screen,
-				 GdkColormap *colormap)
-{
-  GdkColormap *old_colormap;
-  
-  g_return_if_fail (GDK_IS_SCREEN (screen));
-  g_return_if_fail (GDK_IS_COLORMAP (colormap));
-
-  old_colormap = GDK_SCREEN_X11 (screen)->default_colormap;
-
-  GDK_SCREEN_X11 (screen)->default_colormap = g_object_ref (colormap);
-  
-  if (old_colormap)
-    g_object_unref (old_colormap);
-}
-
 static void
 _gdk_screen_x11_events_uninit (GdkScreen *screen)
 {
@@ -298,27 +255,19 @@ static void
 gdk_screen_x11_dispose (GObject *object)
 {
   GdkScreenX11 *screen_x11 = GDK_SCREEN_X11 (object);
+  int i;
+
+  for (i = 0; i < 32; ++i)
+    {
+      if (screen_x11->subwindow_gcs[i])
+        {
+          XFreeGC (screen_x11->xdisplay, screen_x11->subwindow_gcs[i]);
+          screen_x11->subwindow_gcs[i] = 0;
+        }
+    }
 
   _gdk_screen_x11_events_uninit (GDK_SCREEN (object));
 
-  if (screen_x11->default_colormap)
-    {
-      g_object_unref (screen_x11->default_colormap);
-      screen_x11->default_colormap = NULL;
-    }
-
-  if (screen_x11->system_colormap)
-    {
-      g_object_unref (screen_x11->system_colormap);
-      screen_x11->system_colormap = NULL;
-    }
-
-  if (screen_x11->rgba_colormap)
-    {
-      g_object_unref (screen_x11->rgba_colormap);
-      screen_x11->rgba_colormap = NULL;
-    }
-
   if (screen_x11->root_window)
     _gdk_window_destroy (screen_x11->root_window, TRUE);
 
@@ -340,9 +289,6 @@ gdk_screen_x11_finalize (GObject *object)
   if (screen_x11->root_window)
     g_object_unref (screen_x11->root_window);
 
-  if (screen_x11->renderer)
-    g_object_unref (screen_x11->renderer);
-
   /* Visual Part */
   for (i = 0; i < screen_x11->nvisuals; i++)
     g_object_unref (screen_x11->visuals[i]);
@@ -351,8 +297,6 @@ gdk_screen_x11_finalize (GObject *object)
 
   g_free (screen_x11->window_manager_name);
 
-  g_hash_table_destroy (screen_x11->colormap_hash);
-
   deinit_multihead (GDK_SCREEN (object));
   
   G_OBJECT_CLASS (_gdk_screen_x11_parent_class)->finalize (object);
@@ -504,9 +448,9 @@ gdk_x11_screen_get_monitor_output (GdkScreen *screen,
 
 /**
  * gdk_screen_get_monitor_geometry:
- * @screen : a #GdkScreen
+ * @screen: a #GdkScreen
  * @monitor_num: the monitor number, between 0 and gdk_screen_get_n_monitors (screen)
- * @dest : a #GdkRectangle to be filled with the monitor geometry
+ * @dest: a #GdkRectangle to be filled with the monitor geometry
  *
  * Retrieves the #GdkRectangle representing the size and position of
  * the individual monitor within the entire screen area.
@@ -532,11 +476,11 @@ gdk_screen_get_monitor_geometry (GdkScreen    *screen,
 }
 
 /**
- * gdk_screen_get_rgba_colormap:
- * @screen: a #GdkScreen.
+ * gdk_screen_get_rgba_visual:
+ * @screen: a #GdkScreen
  * 
- * Gets a colormap to use for creating windows or pixmaps with an
- * alpha channel. The windowing system on which GTK+ is running
+ * Gets a visual to use for creating windows with an alpha channel.
+ * The windowing system on which GTK+ is running
  * may not support this capability, in which case %NULL will
  * be returned. Even if a non-%NULL value is returned, its
  * possible that the window's alpha channel won't be honored
@@ -548,38 +492,6 @@ gdk_screen_get_monitor_geometry (GdkScreen    *screen,
  *
  * For setting an overall opacity for a top-level window, see
  * gdk_window_set_opacity().
-
- * Return value: (transfer none): a colormap to use for windows with
- *     an alpha channel or %NULL if the capability is not available.
- *
- * Since: 2.8
- **/
-GdkColormap *
-gdk_screen_get_rgba_colormap (GdkScreen *screen)
-{
-  GdkScreenX11 *screen_x11;
-
-  g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
-
-  screen_x11 = GDK_SCREEN_X11 (screen);
-
-  if (!screen_x11->rgba_visual)
-    return NULL;
-
-  if (!screen_x11->rgba_colormap)
-    screen_x11->rgba_colormap = gdk_colormap_new (screen_x11->rgba_visual,
-						  FALSE);
-  
-  return screen_x11->rgba_colormap;
-}
-
-/**
- * gdk_screen_get_rgba_visual:
- * @screen: a #GdkScreen
- * 
- * Gets a visual to use for creating windows or pixmaps with an
- * alpha channel. See the docs for gdk_screen_get_rgba_colormap()
- * for caveats.
  * 
  * Return value: (transfer none): a visual to use for windows with an
  *     alpha channel or %NULL if the capability is not available.
@@ -735,6 +647,7 @@ free_monitors (GdkX11Monitor *monitors,
   g_free (monitors);
 }
 
+#ifdef HAVE_RANDR
 static int
 monitor_compare_function (GdkX11Monitor *monitor1,
                           GdkX11Monitor *monitor2)
@@ -759,6 +672,7 @@ monitor_compare_function (GdkX11Monitor *monitor1,
 
   return 0;
 }
+#endif
 
 static gboolean
 init_randr13 (GdkScreen *screen)
@@ -1184,7 +1098,9 @@ _gdk_x11_screen_size_changed (GdkScreen *screen,
 			      XEvent    *event)
 {
   gint width, height;
+#ifdef HAVE_RANDR
   GdkDisplayX11 *display_x11;
+#endif
 
   width = gdk_screen_get_width (screen);
   height = gdk_screen_get_height (screen);
@@ -1204,7 +1120,7 @@ _gdk_x11_screen_size_changed (GdkScreen *screen,
     {
       XConfigureEvent *rcevent = (XConfigureEvent *) event;
       Screen	    *xscreen = gdk_x11_screen_get_xscreen (screen);
-      
+
       xscreen->width   = rcevent->width;
       xscreen->height  = rcevent->height;
     }
@@ -1251,12 +1167,12 @@ _gdk_x11_screen_process_owner_change (GdkScreen *screen,
 
 /**
  * _gdk_windowing_substitute_screen_number:
- * @display_name : The name of a display, in the form used by 
- *                 gdk_display_open (). If %NULL a default value
- *                 will be used. On X11, this is derived from the DISPLAY
- *                 environment variable.
- * @screen_number : The number of a screen within the display
- *                  referred to by @display_name.
+ * @display_name: The name of a display, in the form used by 
+ *                gdk_display_open (). If %NULL a default value
+ *                will be used. On X11, this is derived from the DISPLAY
+ *                environment variable.
+ * @screen_number: The number of a screen within the display
+ *                 referred to by @display_name.
  *
  * Modifies a @display_name to make @screen_number the default
  * screen when the display is opened.
@@ -1381,7 +1297,7 @@ gdk_screen_get_active_window (GdkScreen *screen)
 }
 
 /**
- * gdk_screen_get_window_stack
+ * gdk_screen_get_window_stack:
  * @screen: a #GdkScreen
  *
  * Returns a #GList of #GdkWindow<!-- -->s representing the current
@@ -1400,7 +1316,8 @@ gdk_screen_get_active_window (GdkScreen *screen)
  * windows it contains, so it should be freed using g_list_free() and
  * its windows unrefed using g_object_unref() when no longer needed.
  *
- * Return value: a list of #GdkWindow<!-- -->s for the current window stack,
+ * Return value: (transfer full) (element-type GdkWindow):
+ *     a list of #GdkWindow<!-- -->s for the current window stack,
  *               or %NULL.
  *
  * Since: 2.10
@@ -1511,7 +1428,7 @@ gdk_event_send_client_message_to_all_recurse (GdkDisplay *display,
   result = send || found;
 
  out:
-  gdk_error_trap_pop ();
+  gdk_error_trap_pop_ignored ();
 
   return result;
 }
@@ -1744,7 +1661,6 @@ fetch_net_wm_check_window (GdkScreen *screen)
 
   /* Find out if this WM goes away, so we can reset everything. */
   XSelectInput (screen_x11->xdisplay, *xwindow, StructureNotifyMask);
-  gdk_display_sync (display);
 
   error = gdk_error_trap_pop ();
   if (!error)
@@ -2072,9 +1988,7 @@ gdk_x11_screen_get_window_manager_name (GdkScreen *screen)
                               &n_items, &bytes_after,
                               (guchar **)&name);
 
-          gdk_display_sync (screen_x11->display);
-
-          gdk_error_trap_pop ();
+          gdk_error_trap_pop_ignored ();
 
           if (name != NULL)
             {
@@ -2087,6 +2001,3 @@ gdk_x11_screen_get_window_manager_name (GdkScreen *screen)
 
   return GDK_SCREEN_X11 (screen)->window_manager_name;
 }
-
-#define __GDK_SCREEN_X11_C__
-#include "gdkaliasdef.c"
