@@ -39,6 +39,7 @@
 #endif /* G_OS_WIN32 */
 
 #include "gtkicontheme.h"
+#include "gtkdebug.h"
 #include "gtkiconfactory.h"
 #include "gtkiconcache.h"
 #include "gtkbuiltincache.h"
@@ -109,12 +110,6 @@ struct _GtkIconInfo
   /* Information about the source
    */
   gchar *filename;
-#if defined (G_OS_WIN32) && !defined (_WIN64)
-  /* System codepage version of filename, for DLL ABI backward
-   * compatibility functions.
-   */
-  gchar *cp_filename;
-#endif
   GLoadableIcon *loadable;
   GSList *emblem_infos;
 
@@ -744,7 +739,7 @@ gtk_icon_theme_set_search_path (GtkIconTheme *icon_theme,
 /**
  * gtk_icon_theme_get_search_path:
  * @icon_theme: a #GtkIconTheme
- * @path: (allow-none): (array length=n_elements) (out): location to store a list of icon theme path directories or %NULL
+ * @path: (allow-none) (array length=n_elements) (out): location to store a list of icon theme path directories or %NULL
  *        The stored value should be freed with g_strfreev().
  * @n_elements: location to store number of elements
  *              in @path, or %NULL
@@ -1322,10 +1317,6 @@ choose_icon (GtkIconTheme       *icon_theme,
 	icon_info->filename = g_strdup (unthemed_icon->svg_filename);
       else if (unthemed_icon->no_svg_filename)
 	icon_info->filename = g_strdup (unthemed_icon->no_svg_filename);
-#if defined (G_OS_WIN32) && !defined (_WIN64)
-      icon_info->cp_filename = g_locale_from_utf8 (icon_info->filename,
-						   -1, NULL, NULL, NULL);
-#endif
 
       icon_info->dir_type = ICON_THEME_DIR_UNTHEMED;
       icon_info->dir_size = size;
@@ -1495,27 +1486,28 @@ gtk_icon_theme_error_quark (void)
  * @icon_theme: a #GtkIconTheme
  * @icon_name: the name of the icon to lookup
  * @size: the desired icon size. The resulting icon may not be
- *        exactly this size; see gtk_icon_info_load_icon().
+ *     exactly this size; see gtk_icon_info_load_icon().
  * @flags: flags modifying the behavior of the icon lookup
- * @error: (allow-none): Location to store error information on failure, or %NULL.
- * 
+ * @error: (allow-none): Location to store error information on failure,
+ *     or %NULL.
+ *
  * Looks up an icon in an icon theme, scales it to the given size
  * and renders it into a pixbuf. This is a convenience function;
  * if more details about the icon are needed, use
  * gtk_icon_theme_lookup_icon() followed by gtk_icon_info_load_icon().
  *
  * Note that you probably want to listen for icon theme changes and
- * update the icon. This is usually done by connecting to the 
+ * update the icon. This is usually done by connecting to the
  * GtkWidget::style-set signal. If for some reason you do not want to
  * update the icon when the icon theme changes, you should consider
  * using gdk_pixbuf_copy() to make a private copy of the pixbuf
- * returned by this function. Otherwise GTK+ may need to keep the old 
+ * returned by this function. Otherwise GTK+ may need to keep the old
  * icon theme loaded, which would be a waste of memory.
- * 
- * Return value: the rendered icon; this may be a newly created icon
- *  or a new reference to an internal icon, so you must not modify
- *  the icon. Use g_object_unref() to release your reference to the
- *  icon. %NULL if the icon isn't found.
+ *
+ * Return value: (transfer full): the rendered icon; this may be a
+ *     newly created icon or a new reference to an internal icon, so
+ *     you must not modify the icon. Use g_object_unref() to release
+ *     your reference to the icon. %NULL if the icon isn't found.
  *
  * Since: 2.4
  **/
@@ -2209,17 +2201,10 @@ theme_lookup_icon (IconTheme          *theme,
           file = g_strconcat (icon_name, string_from_suffix (suffix), NULL);
           icon_info->filename = g_build_filename (min_dir->dir, file, NULL);
           g_free (file);
-#if defined (G_OS_WIN32) && !defined (_WIN64)
-          icon_info->cp_filename = g_locale_from_utf8 (icon_info->filename,
-						   -1, NULL, NULL, NULL);
-#endif
         }
       else
         {
           icon_info->filename = NULL;
-#if defined (G_OS_WIN32) && !defined (_WIN64)
-          icon_info->cp_filename = NULL;
-#endif
         }
       
       if (min_dir->icon_data != NULL)
@@ -2598,19 +2583,10 @@ icon_data_free (GtkIconData *icon_data)
 /*
  * GtkIconInfo
  */
-GType
-gtk_icon_info_get_type (void)
-{
-  static GType our_type = 0;
-  
-  if (our_type == 0)
-    our_type = g_boxed_type_register_static (I_("GtkIconInfo"),
-					     (GBoxedCopyFunc) gtk_icon_info_copy,
-					     (GBoxedFreeFunc) gtk_icon_info_free);
 
-
-  return our_type;
-}
+G_DEFINE_BOXED_TYPE (GtkIconInfo, gtk_icon_info,
+                     gtk_icon_info_copy,
+                     gtk_icon_info_free)
 
 static GtkIconInfo *
 icon_info_new (void)
@@ -2675,9 +2651,6 @@ gtk_icon_info_free (GtkIconInfo *icon_info)
     return;
  
   g_free (icon_info->filename);
-#if defined (G_OS_WIN32) && !defined (_WIN64)
-  g_free (icon_info->cp_filename);
-#endif
   if (icon_info->loadable)
     g_object_unref (icon_info->loadable);
   g_slist_foreach (icon_info->emblem_infos, (GFunc)gtk_icon_info_free, NULL);
@@ -3035,8 +3008,9 @@ icon_info_ensure_scale_and_pixbuf (GtkIconInfo  *icon_info,
 /**
  * gtk_icon_info_load_icon:
  * @icon_info: a #GtkIconInfo structure from gtk_icon_theme_lookup_icon()
- * @error: (allow-none): location to store error information on failure, or %NULL.
- * 
+ * @error: (allow-none): location to store error information on failure,
+ *     or %NULL.
+ *
  * Renders an icon previously looked up in an icon theme using
  * gtk_icon_theme_lookup_icon(); the size will be based on the size
  * passed to gtk_icon_theme_lookup_icon(). Note that the resulting
@@ -3047,12 +3021,12 @@ icon_info_ensure_scale_and_pixbuf (GtkIconInfo  *icon_info,
  * up too far. (This maintains sharpness.). This behaviour can be changed
  * by passing the %GTK_ICON_LOOKUP_FORCE_SIZE flag when obtaining
  * the #GtkIconInfo. If this flag has been specified, the pixbuf
- * returned by this function will be scaled to the exact size. 
- * 
- * Return value: the rendered icon; this may be a newly created icon
- *  or a new reference to an internal icon, so you must not modify
- *  the icon. Use g_object_unref() to release your reference to the
- *  icon.
+ * returned by this function will be scaled to the exact size.
+ *
+ * Return value: (transfer full): the rendered icon; this may be a newly
+ *     created icon or a new reference to an internal icon, so you must
+ *     not modify the icon. Use g_object_unref() to release your reference
+ *     to the icon.
  *
  * Since: 2.4
  **/
@@ -3088,15 +3062,90 @@ gdk_color_to_css (GdkColor *color)
                           color->blue >> 8);
 }
 
+static GdkPixbuf *
+_gtk_icon_info_load_symbolic_internal (GtkIconInfo  *icon_info,
+                                       const gchar  *css_fg,
+                                       const gchar  *css_success,
+                                       const gchar  *css_warning,
+                                       const gchar  *css_error,
+                                       GError      **error)
+{
+  GInputStream *stream;
+  GdkPixbuf *pixbuf;
+  gchar *data;
+  gchar *success, *warning, *err;
+
+  /* css_fg can't possibly have failed, otherwise
+   * that would mean we have a broken style */
+  g_return_val_if_fail (css_fg != NULL, NULL);
+
+  success = warning = err = NULL;
+
+  if (!css_success)
+    {
+      GdkColor success_default_color = { 0, 0x4e00, 0x9a00, 0x0600 };
+      success = gdk_color_to_css (&success_default_color);
+    }
+  if (!css_warning)
+    {
+      GdkColor warning_default_color = { 0, 0xf500, 0x7900, 0x3e00 };
+      warning = gdk_color_to_css (&warning_default_color);
+    }
+  if (!css_error)
+    {
+      GdkColor error_default_color = { 0, 0xcc00, 0x0000, 0x0000 };
+      err = gdk_color_to_css (&error_default_color);
+    }
+
+
+  data = g_strconcat ("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n"
+                      "<svg version=\"1.1\"\n"
+                      "     xmlns=\"http://www.w3.org/2000/svg\"\n"
+                      "     xmlns:xi=\"http://www.w3.org/2001/XInclude\"\n"
+                      "     width=\"16\"\n"
+                      "     height=\"16\">\n"
+                      "  <style type=\"text/css\">\n"
+                      "    rect,path {\n"
+                      "      fill: ", css_fg," !important;\n"
+                      "    }\n"
+                      "    .warning {\n"
+                      "      fill: ", css_warning ? css_warning : warning," !important;\n"
+                      "    }\n"
+                      "    .error {\n"
+                      "      fill: ", css_error ? css_error : err," !important;\n"
+                      "    }\n"
+                      "    .success {\n"
+                      "      fill: ", css_success ? css_success : success," !important;\n"
+                      "    }\n"
+                      "  </style>\n"
+                      "  <xi:include href=\"", icon_info->filename, "\"/>\n"
+                      "</svg>",
+                      NULL);
+  g_free (warning);
+  g_free (err);
+  g_free (success);
+
+  stream = g_memory_input_stream_new_from_data (data, -1, g_free);
+  pixbuf = gdk_pixbuf_new_from_stream_at_scale (stream,
+                                                icon_info->desired_size,
+                                                icon_info->desired_size,
+                                                TRUE,
+                                                NULL,
+                                                error);
+  g_object_unref (stream);
+
+  return pixbuf;
+}
+
 /**
  * gtk_icon_info_load_symbolic:
  * @icon_info: a #GtkIconInfo
- * @fg: a #GdkColor representing the foreground color of the icon
- * @success_color: (allow-none): a #GdkColor representing the warning color
+ * @fg: a #GdkRGBA representing the foreground color of the icon
+ * @success_color: (allow-none): a #GdkRGBA representing the warning color
  *     of the icon or %NULL to use the default color
- * @warning_color: (allow-none): a #GdkColor representing the warning color
+ * @warning_color: (allow-none): a #GdkRGBA representing the warning color
  *     of the icon or %NULL to use the default color
- * @error_color: (allow-none): a #GdkColor representing the error color
+ * @error_color: (allow-none): a #GdkRGBA representing the error color
  *     of the icon or %NULL to use the default color (allow-none)
  * @was_symbolic: (allow-none): a #gboolean, returns whether the loaded icon
  *     was a symbolic one and whether the @fg color was applied to it.
@@ -3120,22 +3169,20 @@ gdk_color_to_css (GdkColor *color)
  * See the <ulink url="http://www.freedesktop.org/wiki/SymbolicIcons">Symbolic Icons spec</ulink>
  * for more information about symbolic icons.
  *
- * Return value: a #GdkPixbuf representing the loaded icon
+ * Return value: (transfer full): a #GdkPixbuf representing the loaded icon
  *
  * Since: 3.0
  **/
 GdkPixbuf *
 gtk_icon_info_load_symbolic (GtkIconInfo  *icon_info,
-                             GdkColor     *fg,
-                             GdkColor     *success_color,
-                             GdkColor     *warning_color,
-                             GdkColor     *error_color,
+                             GdkRGBA      *fg,
+                             GdkRGBA      *success_color,
+                             GdkRGBA      *warning_color,
+                             GdkRGBA      *error_color,
                              gboolean     *was_symbolic,
                              GError      **error)
 {
   GdkPixbuf *pixbuf;
-  GInputStream *stream;
-  gchar *data;
   gchar *css_fg;
   gchar *css_success;
   gchar *css_warning;
@@ -3154,68 +3201,28 @@ gtk_icon_info_load_symbolic (GtkIconInfo  *icon_info,
   if (was_symbolic)
     *was_symbolic = TRUE;
 
-  css_fg = gdk_color_to_css (fg);
-  if (!warning_color)
-    {
-      GdkColor warning_default_color = { 0, 0xf500, 0x7900, 0x3e00 };
-      css_warning = gdk_color_to_css (&warning_default_color);
-    }
-  else
-      css_warning = gdk_color_to_css (warning_color);
-  if (!error_color)
-    {
-      GdkColor error_default_color = { 0, 0xcc00, 0x0000, 0x0000 };
-      css_error = gdk_color_to_css (&error_default_color);
-    }
-  else
-      css_error = gdk_color_to_css (error_color);
-  if (!success_color)
-    {
-      GdkColor success_default_color = { 0, 0x4e00, 0x9a00, 0x0600 };
-      css_success = gdk_color_to_css (&success_default_color);
-    }
-  else
-      css_success = gdk_color_to_css (success_color);
+  css_fg = gdk_rgba_to_string (fg);
 
-  data = g_strconcat (
-"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n"
-"<svg version=\"1.1\"\n"
-"     xmlns=\"http://www.w3.org/2000/svg\"\n"
-"     xmlns:xi=\"http://www.w3.org/2001/XInclude\"\n"
-"     width=\"16\"\n"
-"     height=\"16\">\n"
-"  <style type=\"text/css\">\n"
-"    rect,path {\n"
-"      fill: ", css_fg," !important;\n"
-"    }\n"
-"    .warning {\n"
-"      fill: ", css_warning," !important;\n"
-"    }\n"
-"    .error {\n"
-"      fill: ", css_error," !important;\n"
-"    }\n"
-"    .success {\n"
-"      fill: ", css_success," !important;\n"
-"    }\n"
-"  </style>\n"
-"  <xi:include href=\"", icon_info->filename, "\"/>\n"
-"</svg>",
-         NULL);
+  css_success = css_warning = css_error = NULL;
+
+  if (warning_color)
+    css_warning = gdk_rgba_to_string (warning_color);
+
+  if (error_color)
+    css_error = gdk_rgba_to_string (error_color);
+
+  if (success_color)
+    css_success = gdk_rgba_to_string (success_color);
+
+  pixbuf = _gtk_icon_info_load_symbolic_internal (icon_info,
+                                                  css_fg, css_success,
+                                                  css_warning, css_error,
+                                                  error);
   g_free (css_fg);
   g_free (css_warning);
   g_free (css_success);
   g_free (css_error);
 
-  stream = g_memory_input_stream_new_from_data (data, -1, g_free);
-
-  pixbuf = gdk_pixbuf_new_from_stream_at_scale (stream,
-                                                icon_info->desired_size,
-                                                icon_info->desired_size,
-                                                TRUE,
-                                                NULL,
-                                                error);
-  g_object_unref (stream);
-
   return pixbuf;
 }
 
@@ -3237,7 +3244,7 @@ gtk_icon_info_load_symbolic (GtkIconInfo  *icon_info,
  *
  * See gtk_icon_info_load_symbolic() for more details.
  *
- * Return value: a #GdkPixbuf representing the loaded icon
+ * Return value: (transfer full): a #GdkPixbuf representing the loaded icon
  *
  * Since: 3.0
  **/
@@ -3248,24 +3255,50 @@ gtk_icon_info_load_symbolic_for_style (GtkIconInfo   *icon_info,
                                        gboolean      *was_symbolic,
                                        GError       **error)
 {
+  GdkPixbuf *pixbuf;
   GdkColor success_color;
   GdkColor warning_color;
   GdkColor error_color;
   GdkColor *fg;
-  GdkColor *success = NULL;
-  GdkColor *warning = NULL;
-  GdkColor *err = NULL;
+  gchar *css_fg, *css_success;
+  gchar *css_warning, *css_error;
+
+  if (!icon_info->filename ||
+      !g_str_has_suffix (icon_info->filename, "-symbolic.svg"))
+    {
+      if (was_symbolic)
+        *was_symbolic = FALSE;
+      return gtk_icon_info_load_icon (icon_info, error);
+    }
+
+  if (was_symbolic)
+    *was_symbolic = TRUE;
 
   fg = &style->fg[state];
-  if (gtk_style_lookup_color (style, "success_color", &success_color))
-    success = &success_color;
-  if (gtk_style_lookup_color (style, "warning_color", &warning_color))
-    warning = &warning_color;
-  if (gtk_style_lookup_color (style, "error_color", &error_color))
-    err = &error_color;
+  css_fg = gdk_color_to_css (fg);
 
-  return gtk_icon_info_load_symbolic (icon_info, fg, success, warning, err,
-                                      was_symbolic, error);
+  css_success = css_warning = css_error = NULL;
+
+  if (gtk_style_lookup_color (style, "success_color", &success_color))
+    css_success = gdk_color_to_css (&success_color);
+
+  if (gtk_style_lookup_color (style, "warning_color", &warning_color))
+    css_warning = gdk_color_to_css (&warning_color);
+
+  if (gtk_style_lookup_color (style, "error_color", &error_color))
+    css_error = gdk_color_to_css (&error_color);
+
+  pixbuf = _gtk_icon_info_load_symbolic_internal (icon_info,
+                                                  css_fg, css_success,
+                                                  css_warning, css_error,
+                                                  error);
+
+  g_free (css_fg);
+  g_free (css_success);
+  g_free (css_warning);
+  g_free (css_error);
+
+  return pixbuf;
 }
 
 /**
@@ -3381,7 +3414,7 @@ gtk_icon_info_get_embedded_rect (GtkIconInfo  *icon_info,
 /**
  * gtk_icon_info_get_attach_points:
  * @icon_info: a #GtkIconInfo
- * @points: (allow-none): (array length=n_points) (out): location to store pointer to an array of points, or %NULL
+ * @points: (allow-none) (array length=n_points) (out): location to store pointer to an array of points, or %NULL
  *          free the array of points with g_free().
  * @n_points: (allow-none): location to store the number of points in @points, or %NULL
  * 
@@ -3719,94 +3752,3 @@ gtk_icon_info_new_for_pixbuf (GtkIconTheme *icon_theme,
 
   return info;
 }
-
-#if defined (G_OS_WIN32) && !defined (_WIN64)
-
-/* DLL ABI stability backward compatibility versions */
-
-#undef gtk_icon_theme_set_search_path
-
-void
-gtk_icon_theme_set_search_path (GtkIconTheme *icon_theme,
-				const gchar  *path[],
-				gint          n_elements)
-{
-  const gchar **utf8_path;
-  gint i;
-
-  utf8_path = g_new (const gchar *, n_elements);
-
-  for (i = 0; i < n_elements; i++)
-    utf8_path[i] = g_locale_to_utf8 (path[i], -1, NULL, NULL, NULL);
-
-  gtk_icon_theme_set_search_path_utf8 (icon_theme, utf8_path, n_elements);
-
-  for (i = 0; i < n_elements; i++)
-    g_free ((gchar *) utf8_path[i]);
-
-  g_free (utf8_path);
-}
-
-#undef gtk_icon_theme_get_search_path
-
-void
-gtk_icon_theme_get_search_path (GtkIconTheme      *icon_theme,
-				gchar            **path[],
-				gint              *n_elements)
-{
-  gint i, n;
-
-  gtk_icon_theme_get_search_path_utf8 (icon_theme, path, &n);
-
-  if (n_elements)
-    *n_elements = n;
-
-  if (path)
-    {
-      for (i = 0; i < n; i++)
-	{
-	  gchar *tem = (*path)[i];
-
-	  (*path)[i] = g_locale_from_utf8 ((*path)[i], -1, NULL, NULL, NULL);
-	  g_free (tem);
-	}
-    }
-}
-
-#undef gtk_icon_theme_append_search_path
-
-void
-gtk_icon_theme_append_search_path (GtkIconTheme *icon_theme,
-				   const gchar  *path)
-{
-  gchar *utf8_path = g_locale_from_utf8 (path, -1, NULL, NULL, NULL);
-
-  gtk_icon_theme_append_search_path_utf8 (icon_theme, utf8_path);
-
-  g_free (utf8_path);
-}
-
-#undef gtk_icon_theme_prepend_search_path
-
-void
-gtk_icon_theme_prepend_search_path (GtkIconTheme *icon_theme,
-				    const gchar  *path)
-{
-  gchar *utf8_path = g_locale_from_utf8 (path, -1, NULL, NULL, NULL);
-
-  gtk_icon_theme_prepend_search_path_utf8 (icon_theme, utf8_path);
-
-  g_free (utf8_path);
-}
-
-#undef gtk_icon_info_get_filename
-
-G_CONST_RETURN gchar *
-gtk_icon_info_get_filename (GtkIconInfo *icon_info)
-{
-  g_return_val_if_fail (icon_info != NULL, NULL);
-
-  return icon_info->cp_filename;
-}
-
-#endif
