@@ -78,9 +78,11 @@ window_creation_function (GtkNotebook *source_notebook,
 
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   notebook = gtk_notebook_new ();
+  g_signal_connect (notebook, "create-window",
+                    G_CALLBACK (window_creation_function), NULL);
 
-  gtk_notebook_set_group (GTK_NOTEBOOK (notebook),
-			  gtk_notebook_get_group (source_notebook));
+  gtk_notebook_set_group_name (GTK_NOTEBOOK (notebook),
+			       gtk_notebook_get_group_name (source_notebook));
 
   gtk_container_add (GTK_CONTAINER (window), notebook);
 
@@ -142,7 +144,7 @@ on_button_drag_data_received (GtkWidget        *widget,
 
 static GtkWidget*
 create_notebook (gchar           **labels,
-		 gpointer          group,
+		 const gchar      *group,
 		 gint              packing,
 		 GtkPositionType   pos)
 {
@@ -150,11 +152,13 @@ create_notebook (gchar           **labels,
   gint count = 0;
 
   notebook = gtk_notebook_new ();
+  g_signal_connect (notebook, "create-window",
+                    G_CALLBACK (window_creation_function), NULL);
 
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), pos);
   gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
   gtk_container_set_border_width (GTK_CONTAINER (notebook), 6);
-  gtk_notebook_set_group (GTK_NOTEBOOK (notebook), group);
+  gtk_notebook_set_group_name (GTK_NOTEBOOK (notebook), group);
 
   while (*labels)
     {
@@ -184,7 +188,7 @@ create_notebook (gchar           **labels,
 
 static GtkWidget*
 create_notebook_with_notebooks (gchar           **labels,
-			        gpointer          group,
+			        const gchar      *group,
 			        gint              packing,
 			        GtkPositionType   pos)
 {
@@ -192,11 +196,13 @@ create_notebook_with_notebooks (gchar           **labels,
   gint count = 0;
 
   notebook = gtk_notebook_new ();
+  g_signal_connect (notebook, "create-window",
+                    G_CALLBACK (window_creation_function), NULL);
 
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), pos);
   gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
   gtk_container_set_border_width (GTK_CONTAINER (notebook), 6);
-  gtk_notebook_set_group (GTK_NOTEBOOK (notebook), group);
+  gtk_notebook_set_group_name (GTK_NOTEBOOK (notebook), group);
 
   while (*labels)
     {
@@ -252,8 +258,6 @@ main (gint argc, gchar *argv[])
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   table = gtk_table_new (3, 2, FALSE);
 
-  gtk_notebook_set_window_creation_hook (window_creation_function, NULL, NULL);
-
   gtk_table_attach_defaults (GTK_TABLE (table),
 			     create_notebook (tabs1, GROUP_A, PACK_ALTERNATE, GTK_POS_TOP),
 			     0, 1, 0, 1);
